@@ -89,6 +89,8 @@ const DesktopManager = class {
         this._clickY = null;
         this._compositeStackList = null;
         this._displayList = [];
+        // Track last seen keyboard modifier state so accelerators can query it
+        this._lastModifierState = 0;
         this.ignoreKeys = this.Enums.IgnoreKeys.map(_k => Gdk[_k]);
 
         // setup gracefull termination
@@ -306,7 +308,7 @@ const DesktopManager = class {
         if (button === 3)
             this.mainApp.activate_action('displayShellBackgroundMenu', null);
     }
-    
+
     onWidgetDisplayChanged() {
         if (this.Prefs.showDesktopWidgets)
             this._startWidgetDisplay();
@@ -316,7 +318,8 @@ const DesktopManager = class {
 
     _startWidgetDisplay() {
         this.widgetManager
-        .startWidgetDisplay(this._desktops, {redisplay: true});
+        .startWidgetDisplay(this._desktops, {redisplay: true})
+        .catch(e => logError(e));
     }
 
     _stopWidgetDisplay() {
@@ -383,6 +386,16 @@ const DesktopManager = class {
         }
     }
 
+    updateModifierState(state) {
+        // cache modifier state for use by accelerator-driven handlers
+        this._lastModifierState = state;
+    }
+
+    clearModifierState() {
+        // reset cached modifier state
+        this._lastModifierState = 0;
+    }
+
     closePopUps() {
         if (this._renameWindow) {
             this._renameWindow.close();
@@ -414,7 +427,7 @@ const DesktopManager = class {
             gridschanged,
         });
 
-        this.widgetManager.applyLayoutChange(this._desktops, {redisplay});
+        await this.widgetManager.applyLayoutChange(this._desktops, {redisplay});
     }
 
 
@@ -1512,6 +1525,7 @@ const DesktopManager = class {
     unselectAll() {
         this._displayList.forEach(f => {
             f.unsetSelected();
+            f.keyboardUnSelected();
             f.opacity = 1;
         });
         this.fileItemMenu.activeFileItem = null;
@@ -1843,4 +1857,13 @@ const DesktopManager = class {
     get writableByOthers() {
         return this.desktopMonitor._writableByOthers;
     }
+
+    get modifierMode() {
+        return {
+            state: this._lastModifierState,
+            ctrl: (this._lastModifierState & Gdk.ModifierType.CONTROL_MASK) !== 0,
+            shift: (this._lastModifierState & Gdk.ModifierType.SHIFT_MASK) !== 0,
+            alt: (this._lastModifierState & Gdk.ModifierType.ALT_MASK) !== 0,
+        };
+    }
 };
