@@ -40,10 +40,6 @@ const DragManager = class {
         this.dragItem = null;
         this.rubberBand = false;
         this.localDragOffset = [0, 0];
-        // Remember the anchor item for Shift+click range selections.
-        // This should be set when the user makes an explicit selection
-        // (single click or ctrl-click) and used by Shift selections.
-        this._selectionAnchor = null;
     }
 
     // Drag and Drop local Methods
@@ -81,13 +77,13 @@ const DragManager = class {
             let i = 0;
             let newSymlinkName = fileGio.get_basename();
             let checkSymlinkGio;
+
             do {
                 checkSymlinkGio =
-                    Gio.File.new_for_commandline_arg(
-                        GLib.build_filenamev(
-                            [gioDestination.get_path(), newSymlinkName]
-                        )
+                    Gio.File.new_build_filenamev(
+                        [gioDestination.get_path(), newSymlinkName]
                     );
+
                 try {
                     checkSymlinkGio.make_symbolic_link(
                         GLib.build_filenamev([fileGio.get_path()]),
@@ -275,9 +271,9 @@ const DragManager = class {
         return false;
     }
 
-    _drawSelectionRectangles() {
+    _drawRubberBand() {
         for (let grid of this._desktops)
-            grid.drawRubberBand();
+            grid.updateOverlay();
     }
 
     // Global Methods
@@ -369,15 +365,16 @@ const DragManager = class {
         this.pendingSelfCopyFiles = {};
 
         await Promise.all(fileList.map(async element => {
-            let file = Gio.File.new_for_uri(element);
+            const file = Gio.File.new_for_uri(element);
 
             if (!file.is_native()) {
                 this.setPendingDropCoordinates(file, dropCoordinates);
                 return;
             }
 
-            let info = new Gio.FileInfo();
+            const info = new Gio.FileInfo();
             info.set_attribute_string('metadata::desktop-icon-position', '');
+
             if (dropCoordinates !== null) {
                 if (!opts.doCopy) {
                     info.set_attribute_string(
@@ -427,16 +424,12 @@ const DragManager = class {
 
     // Drag Methods
 
-    startRubberband(X, Y, shiftPressed = false, controlPressed = false) {
+    startRubberband(X, Y) {
         this.rubberBandInitX = X;
         this.rubberBandInitY = Y;
         this.rubberBand = true;
-        this.rubberBandShift = shiftPressed;
-        this.rubberBandControl = controlPressed;
-        for (let item of this._displayList) {
+        for (let item of this._displayList)
             item.touchedByRubberband = false;
-            item.rubberBandInitialState = item.isSelected;
-        }
     }
 
     onDragBegin(item) {
@@ -534,7 +527,7 @@ const DragManager = class {
             if (!fileList)
                 return;
             if (gdkDropAction === Gdk.DragAction.MOVE ||
-                    gdkDropAction === Gdk.DragAction.COPY) {
+                gdkDropAction === Gdk.DragAction.COPY) {
                 try {
                     if (!localDrop) {
                         await this.clearFileCoordinates(
@@ -543,6 +536,7 @@ const DragManager = class {
                             {doCopy: forceCopy}
                         );
                     }
+
                     returnAction = await this.copyOrMoveUris(
                         fileList,
                         this._desktopDir.get_uri(),
@@ -557,6 +551,7 @@ const DragManager = class {
                     returnAction = Gdk.DragAction.LINK;
                 else
                     returnAction = Gdk.DragAction.COPY;
+
                 this.askWhatToDoWithFiles(
                     fileList,
                     this._desktopDir.get_uri(),
@@ -566,7 +561,7 @@ const DragManager = class {
                     ylocalDestination,
                     event
                 )
-                        .catch(e => logError(e));
+                .catch(e => logError(e));
             }
             break;
         case this._Enums.DndTargetInfo.TEXT_PLAIN:
@@ -645,53 +640,19 @@ const DragManager = class {
                     'width': this.x2 - this.x1,
                     'height': this.y2 - this.y1,
                 });
-            this._drawSelectionRectangles();
-            let anyItemTouched = false;
-            let closestItem = null;
-            let closestDistance = null;
-            const toggleMode = this.rubberBandShift && this.rubberBandControl;
-            const unselectMode = this.rubberBandShift && !this.rubberBandControl;
+            this._drawRubberBand();
             for (let item of this._displayList) {
                 const labelintersect =
                     item.labelRectangle.intersect(this.selectionRectangle)[0];
                 const iconintersect =
                     item.iconRectangle.intersect(this.selectionRectangle)[0];
                 if (labelintersect || iconintersect) {
-                    if (toggleMode) {
-                        // Toggle selection based on initial state
-                        if (item.rubberBandInitialState)
-                            item.unsetSelected();
-                        else
-                            item.setSelected();
-                    } else if (unselectMode) {
-                        // Shift only: unselect items in rubberband
-                        item.unsetSelected();
-                    } else {
-                        item.setSelected();
-                    }
+                    item.setSelected();
                     item.touchedByRubberband = true;
-                    anyItemTouched = true;
-                    // Calculate distance from cursor to item center
-                    const [itemX, itemY] = item.getCoordinates();
-                    const itemCenterX = itemX + item.iconRectangle.width / 2;
-                    const itemCenterY = itemY + item.iconRectangle.height / 2;
-                    const distance = Math.pow(X - itemCenterX, 2) + Math.pow(Y - itemCenterY, 2);
-                    if (closestDistance === null || distance < closestDistance) {
-                        closestDistance = distance;
-                        closestItem = item;
-                    }
                 } else if (item.touchedByRubberband) {
-                    // Restore initial state when leaving the rubberband area
-                    if (item.rubberBandInitialState)
-                        item.setSelected();
-                    else
-                        item.unsetSelected();
+                    item.unsetSelected();
                 }
             }
-            if (anyItemTouched) {
-                this._desktopManager.desktopActions._isMassSelectionInProgress = true;
-                this._desktopManager.desktopActions._keyboardHoveredItem = closestItem;
-            }
         }
     }
 
@@ -699,12 +660,9 @@ const DragManager = class {
         if (this.rubberBand) {
             this.rubberBand = false;
             this.selectionRectangle = null;
-            // Reset mass selection flag if no items were selected
-            if (!this.currentSelection || this.currentSelection.length === 0)
-                this._desktopManager.desktopActions._isMassSelectionInProgress = false;
         }
         for (let grid of this._desktops)
-            grid.drawRubberBand();
+            grid.updateOverlay();
 
         return false;
     }
@@ -716,90 +674,20 @@ const DragManager = class {
     }
 
     selected(fileItem, action) {
-        if (!fileItem)
-            return;
-        
+        this._clearKeyboardSelection();
         switch (action) {
         case this._Enums.Selection.ALONE:
-            this._desktopManager.desktopActions._exitMassSelectionMode(fileItem);
-            fileItem.unsetHoveredWithKeyboard();
-            break;
-        case this._Enums.Selection.WITH_SHIFT: {
-            // Use the explicit selection anchor if present; otherwise fall
-            // back to the first selected item in display order.
-            const list = this._displayList;
-            const clickedIndex = list.indexOf(fileItem);
-
-            let anchorIndex = -1;
-            if (this._selectionAnchor)
-                anchorIndex = list.indexOf(this._selectionAnchor);
-
-
-            if (anchorIndex === -1) {
-                // fallback: first selected item in display order
-                for (let i = 0; i < list.length; i++) {
-                    if (list[i].isSelected) {
-                        anchorIndex = i;
-                        break;
-                    }
+            if (!fileItem.isSelected) {
+                for (let item of this._displayList) {
+                    if (item === fileItem)
+                        item.setSelected();
+                    else
+                        item.unsetSelected();
                 }
             }
-
-            if (anchorIndex === -1) {
-                // No anchor and no selection: just select the clicked item.
-                if (clickedIndex !== -1)
-                    list[clickedIndex].setSelected();
-                break;
-            }
-
-            const start = Math.min(anchorIndex, clickedIndex);
-            const end = Math.max(anchorIndex, clickedIndex);
-
-            for (let i = 0; i < list.length; i++) {
-                if (i >= start && i <= end)
-                    list[i].setSelected();
-                else
-                    list[i].unsetSelected();
-            }
             break;
-        }
-        case this._Enums.Selection.WITH_SHIFT_CONTROL: {
-            // Similar to WITH_SHIFT, but preserve existing selections
-            // (i.e. add the range to current selection instead of replacing it).
-            const list = this._displayList;
-            const clickedIndex = list.indexOf(fileItem);
-
-            let anchorIndex = -1;
-            if (this._selectionAnchor)
-                anchorIndex = list.indexOf(this._selectionAnchor);
-
-            if (anchorIndex === -1) {
-                for (let i = 0; i < list.length; i++) {
-                    if (list[i].isSelected) {
-                        anchorIndex = i;
-                        break;
-                    }
-                }
-            }
-
-            if (anchorIndex === -1) {
-                // No anchor: just toggle/select the clicked item
-                if (clickedIndex !== -1)
-                    list[clickedIndex].setSelected();
-                break;
-            }
-
-            const start2 = Math.min(anchorIndex, clickedIndex);
-            const end2 = Math.max(anchorIndex, clickedIndex);
-
-            for (let i = start2; i <= end2; i++)
-                list[i].setSelected();
-
-            break;
-        }
-        case this._Enums.Selection.WITH_CONTROL:
+        case this._Enums.Selection.WITH_SHIFT:
             fileItem.toggleSelected();
-            this._selectionAnchor = fileItem;  
             break;
         case this._Enums.Selection.RIGHT_BUTTON:
             if (!fileItem.isSelected) {
@@ -809,8 +697,6 @@ const DragManager = class {
                     else
                         item.unsetSelected();
                 }
-                // set anchor when selection is changed by right-click
-                this._selectionAnchor = fileItem;
             }
             break;
         case this._Enums.Selection.ENTER:
@@ -831,6 +717,10 @@ const DragManager = class {
         }
     }
 
+    _clearKeyboardSelection() {
+        this._displayList.forEach(item => item.keyboardUnSelected());
+    }
+
     // File Copy Move Link Methods
 
     async copyOrMoveUris(uriList, destinationUri, event, params = {}) {
@@ -997,19 +887,21 @@ const DragManager = class {
 
     async makeLinks(fileList, destination, X, Y) {
         const gioDestination = Gio.File.new_for_uri(destination);
+
         await Promise.all(fileList.map(async file => {
             const fileGio = Gio.File.new_for_uri(file);
+
             const newSymlinkName =
                 this._desktopManager.desktopMonitor
                 .getDesktopUniqueFileName(
                     fileGio.get_basename()
                 );
+
             const symlinkGio =
-                Gio.File.new_for_commandline_arg(
-                    GLib.build_filenamev(
-                        [gioDestination.get_path(), newSymlinkName]
-                    )
+                Gio.File.new_build_filenamev(
+                    [gioDestination.get_path(), newSymlinkName]
                 );
+
             try {
                 const linkMade =
                     symlinkGio.make_symbolic_link(
@@ -1019,10 +911,12 @@ const DragManager = class {
 
                 if (linkMade) {
                     const info = new Gio.FileInfo();
+
                     info.set_attribute_string(
                         'metadata::nautilus-drop-position',
                         `${X},${Y}`
                     );
+
                     info.set_attribute_string(
                         'metadata::desktop-icon-position',
                         ''
@@ -1090,4 +984,3 @@ const DragManager = class {
         return this._localDrag();
     }
 };
-
